
%{
#include "rumd/Sample.h"
#include "rumd/Potential.h"
%}


// the following is to allow proper handling of the default argument "w".
// The string typemap does not seem to be applied when there are default
// arguments and I couldn't figure out how to change that

%feature("shadow") Sample::WriteConf(const std::string&filename, 
				     const std::string& mode) %{
  def WriteConf(self, filename, mode="w"):
      return _rumd.Sample_WriteConf(self, filename, mode)
    %}

%rename (SetOutputManagerMetaData_bool) SetOutputManagerMetaData(const std::string&, const std::string&, bool);

%rename (SetOutputManagerMetaData_int) SetOutputManagerMetaData(const std::string&, const std::string&, int);

%feature("shadow") Sample::SetOutputManagerMetaData(const std::string& manager_name, const std::string& key, bool on) %{
def SetOutputManagerMetaData(self, manager_name, key, value):
    if type(value) == type(True):
        return _rumd.Sample_SetOutputManagerMetaData_bool(self, manager_name, key, value)
    else:
        return _rumd.Sample_SetOutputManagerMetaData_int(self, manager_name, key, value)
  %}

// to avoid warnings about keyword arguments not being possible for overloaded functions
%feature("kwargs", "0") Sample::SetOutputManagerMetaData;


class Sample
{
 public:
  Sample( unsigned int set_particles_per_block, unsigned int set_threads_per_particle);
  Sample* Copy();
  int GetNumberOfParticles() const;
  unsigned int GetNumberOfTypes() const;
  unsigned int GetNumberThisType(unsigned int type) const;
  SortingScheme GetSortingScheme() const;
  std::string GetOutputDirectory() const;

  void SetVerbose(bool vb);
  void SetCheckCudaErrors(bool set_check_cuda_errors);
  void InitializeOutputManagers(unsigned long int timeStepIndex);
  void NotifyOutputManagers(unsigned long int timeStepIndex);
  void TerminateOutputManagers();
  void ReadConf( const std::string& filename );
  void ReadRestartConf(int restartBlock, unsigned int numDigitsBlkIdx=4);
  void WriteConf(const std::string&filename, const std::string& mode);

  void EnableBackup(bool make_backup);
  void SetOutputBlockSize(unsigned long int blockSize);
  void SetOutputManagerActive(const std::string& manager_name, bool active);
  void SetLogLinParameters(const std::string& manager_name, unsigned int base, unsigned int maxInterval);
  void SetOutputManagerMetaData(const std::string& manager_name, const std::string& key, bool on);
  void SetOutputManagerMetaData(const std::string& manager_name, const std::string& key, int value);
  void AddExternalCalculator(ExternalCalculator* calc) {energiesLogLin->AddExternalCalculator(calc)};
  void RemoveExternalCalculator(ExternalCalculator* calc) {energiesLogLin->RemoveExternalCalculator(calc)};
  

  void SetMoleculeData(MoleculeData* md);
  MoleculeData* GetMoleculeData();
  void SetNumberOfDOFs(unsigned int DOFs);
  void UpdateDegreesOfFreedom();
  void SortParticles();
  
  void SetPotential(Potential* potential);
  void AddPotential(Potential* potential);
  void IsotropicScaleSystem( float Rscal );
  void IsotropicScaleSystemCM( float Rscal );
  void AnisotropicScaleSystem( float Rscal, int dir );
  void AffinelyShearSystem( float shear_strain );
  void ScaleVelocities( float factor );
  void SetSimulationBox(SimulationBox* simBox);
  SimulationBox* GetSimulationBox() const;
  void SetIntegrator(Integrator* newItg){ itg = newItg; }

  void SetPB_TP( unsigned int pb, unsigned int tp );
  void SetSortingScheme( SortingScheme ss);
  void SetIncludeKineticStress(bool include_kinetic_stress);

  unsigned int GetParticlesPerBlock();
  unsigned int GetThreadsPerParticle();
  unsigned int GetNumberOfDOFs() const;

  void CalcF(bool calc_stresses = false);
  double GetPotentialEnergy() const;
  double GetVirial() const;
  float GetMass(unsigned int type) const;
  float GetMeanMass() const;

  void SetOutputDirectory(const std::string&filename);
  void AddOutputManager(const std::string& manager_name, LogLinOutputManager* om);
  const Integrator* GetIntegrator() const;
};

%extend Sample {
  void Assign(const Sample& S) {
    (*$self) = S;
  }


  PyObject* GetPositions() {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = PyArray_BYTES((PyArrayObject*) posArray);
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_r[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
  PyObject* GetVelocities()  {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = PyArray_BYTES((PyArrayObject*) posArray);
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_v[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
  PyObject* GetForces()  {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = PyArray_BYTES((PyArrayObject*) posArray);
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_f[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
  PyObject* GetImages()  {
    $self->CopySimulationDataFromDevice();
    const ParticleData& pData = *$self->GetParticleData();
    int len;
    npy_intp dims[2];
    unsigned int nParticles = $self->GetNumberOfParticles();
    dims[0] = nParticles;
    dims[1] = 3;
    PyObject *posArray = PyArray_SimpleNew(2, dims, NPY_FLOAT);
    char *ptr = PyArray_BYTES((PyArrayObject*) posArray);
    len = 3*sizeof(float)/sizeof(char);

    for(int pdx = 0; pdx < nParticles; pdx++)
      {
	memcpy((void*) ptr, (void*)(&(pData.h_im[pdx])), len);
	ptr += len;
      }
    
    return posArray;
  }
  PyObject* GetStress() {
    std::vector<double> stress_vector = $self->GetStress();
    int len;
    npy_intp dims[1] = {6};
    PyObject *stressArray = PyArray_SimpleNew(1, dims, NPY_DOUBLE);

    double* out_ptr = static_cast<double*>(PyArray_DATA((PyArrayObject*)stressArray));
    for(int idx = 0; idx < 6; idx++)
      out_ptr[idx] = stress_vector[idx];

    return stressArray;
  }
};

