import sys
sys.path.insert(0,"../../../Swig")
sys.path.insert(0,"../../../Python")
sys.path.insert(0,"../../../Tools")

import rumdSimulation, rumd, rumd_tools
# it is essential that there is no doubt about which version is actually run!
print "File used to import the rumd module",rumd.__file__
print "File used to import the rumdSimulation module",rumdSimulation.__file__
print "File used to import the rumd_tools module",rumd_tools.__file__

from rumdSimulation import rumdSimulation
from RUMD_Test import RUMD_Test

import string
import os
import shutil

class Test_Restart (RUMD_Test):
    """
    Test that the restart mechanism works correctly
    """
    def __init__(self):
        RUMD_Test.__init__(self)
        self.trajectory_tol["virials"] = 0.05
        self.numParticlesList = [512]
        self.restartIndex = 2

    def RunSimulation(self, mode, numParticles, pb, tp):

        if mode != "short":
            raise ValueError, "Only \"short\" mode available for restart test"
        self.numTimeSteps = 300


        # sample and simulation
        startConfg = "startN%d.xyz.gz" % numParticles
        
        sim = rumdSimulation(startConfg, pb, tp, verbose=False)

        if self.generate_reference_data:
            sim.SetOutputDirectory(self.ref_directory)
        else:
            # copy part of the reference directory
            if not os.access(self.out_directory, os.F_OK):
                os.mkdir(self.out_directory)
            del_files = os.listdir(self.out_directory)
            for fil in del_files:
                os.remove(self.out_directory + "/"+ fil)
            ref_files = os.listdir(self.ref_directory)
            for fil in ref_files:
                if fil.startswith("trajectory"):
                    blockNum = string.atoi(fil[10:-7])
                elif fil.startswith("energies"):
                    blockNum = string.atoi(fil[8:-7])
                elif fil.startswith("restart"):
                    blockNum = string.atoi(fil[7:-7])
                else:
                    continue
                
                if blockNum <= self.restartIndex:
                    shutil.copy(self.ref_directory + "/" + fil, self.out_directory)



            sim.SetOutputDirectory(self.out_directory)

        # integrator
        itg = rumd.IntegratorNVE(timeStep=0.005)

        # Create LJ pair-potential
        potential = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedPotential)
        # Set up parameters for Lennard-Jones interactions
        # i  j   Sig   Eps  Rcut (in units of Sigma_ij)
        potential.SetParams(0, 0, 1.00, 1.00, 2.5)
    
        sim.SetIntegrator(itg)
        sim.SetPotential(potential)

        # tune output depending on the mode
        sim.SetOutputScheduling("energies","linear",interval=1)
        sim.SetOutputScheduling("trajectory","linear", interval=10)
        sim.SetBlockSize(50)

        if self.generate_reference_data:
            sim.Run(self.numTimeSteps)
        else:
            sim.Run(self.numTimeSteps, restartBlock=self.restartIndex)
            

if __name__ == "__main__":

    rumd_test = Test_Restart()

    sys.exit(rumd_test.RunShort( "gen_ref" in sys.argv ))
