#!/bin/csh -f
#PBS -e en_cons.err
#PBS -o en_cons.log
#PBS -l nodes=1:ppn=3
#PBS -l walltime=50:00:00
#PBS -m ae
# Job name:
#PBS -N en_cons_size
#PBS -r n


# if running as PBS job, change into the correct directory (the directory containing this file)
if ($?PBS_O_WORKDIR) then
cd $PBS_O_WORKDIR
endif

set pb = 16
set tp = 1
set num_blocks = 100
@ block_factor = 100


printf "Physics test: energy conservation/system size.\n"


set dt = 0.0064
set cut = 2.5
set w_interval = 400
set block_size = 40000

set dir_list = (N256 N864 N8192)

foreach dir ($dir_list)

echo "entering directory" $dir
cd $dir
(../../../../../UserPrograms/md_pair  --numIOBlocks=${num_blocks} --IOBlockSize=${block_size} --pb=$pb --tp=$tp --dt=${dt} --thermostat=0 --cutoff=${cut} --device=0 --dump_nrg=${w_interval} --writeTraj=0 --write_v_com_x=1 --mom_reset=0 > en_cons${dt}.log) >& en_cons${dt}.err

zcat TrajectoryFiles/energies*.dat.gz > energies${dt}.dat
rm -rf TrajectoryFiles

cd ..


end

grep "Processing time GPU" N*/en_cons*.log > cpu_times.log
echo "To plot the energies, do: xmgrace -block N256/energies0.0064.dat -bxy 0:6 -block N864/energies0.0064.dat -bxy 0:6 -block N8192/energies0.0064.dat -bxy 0:6
"

echo "To calculate energy fluctuations and drift, do: python ../../Analysis.py in each directory N256, N864 and N8192 [results will be in stdE_vs_dt.dat]"


